; Frequency Counter
; Ikagawa/Ikagawa Lab.
; 1999.11.05 Ver.1.00




; r0    always 0x00

; r1    always 0x01

; r2    value gate time
;       0x00   0.01 sec
;       0x01   0.10 sec
;       0x02   1.00 sec
;       0x03  10.00 sec

; r3    value input channel
;       0x00  ch0
;       0x01  ch1
;       0x02  ch2
;       0x03  ch3 (internal use)

; r4    value operation
;       0x00  count
;       0x01  hold




        .device AT90S1200

; interrupt vectors
        .equ    INT0addr         = 0x001
        .equ    OVF0addr         = 0x002
        .equ    ACIaddr          = 0x003

; registers
        .equ    ACSR             = 0x08
        .equ    PIND             = 0x10
        .equ    DDRD             = 0x11
        .equ    PORTD            = 0x12
        .equ    PINB             = 0x16
        .equ    DDRB             = 0x17
        .equ    PORTB            = 0x18
        .equ    EECR             = 0x1c
        .equ    EEDR             = 0x1d
        .equ    EEAR             = 0x1e
        .equ    WDTCR            = 0x21
        .equ    TCNT0            = 0x32
        .equ    TCCR0            = 0x33
        .equ    MCUCR            = 0x35
        .equ    TIFR             = 0x38
        .equ    TIMSK            = 0x39
        .equ    GIMSK            = 0x3b
        .equ    SREG             = 0x3f

; constants
        .equ    GateTime_10ms    = 0x00
        .equ    GateTime_100ms   = 0x01
        .equ    GateTime_1000ms  = 0x02
        .equ    GateTime_10000ms = 0x03

        .equ    InputChannel_0   = 0x00
        .equ    InputChannel_1   = 0x01
        .equ    InputChannel_2   = 0x02
        .equ    InputChannel_3   = 0x03

        .equ    Operation_Count  = 0x00
        .equ    Operation_Hold   = 0x01




        .cseg

        .org 0x0000
        rjmp    RESET                   ; Reset handler

        .org INT0addr
        reti                            ; External interrupt0 handler

        .org OVF0addr
        reti                            ; Timer0 overflow interrupt handler

        .org ACIaddr
        reti                            ; Analog comparator interrupt handler




RESET:

PortInitialize:
        ldi     r16,0b11111111          ; portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     DDRB,r16                ; i/o    out out out out out out out out

        ldi     r16,0b00000000          ; portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     PORTB,r16               ; data   0   0   0   0   0   0   0   0

        ldi     r16,0b11101111          ; portd  d7  d6  d5  d4  d3  d2  d1  d0
        out     DDRD,r16                ; i/o    x   out out in  out out out out

        ldi     r16,0b00010000          ; portd  d7  d6  d5  d4  d3  d2  d1  d0
        out     PORTD,r16               ; data   0   0   0   1   0   0   0   0




Counter0Initialize:
        ldi     r16,0b00000110          ; Counter0.Clk=External Pin T0, falling edge
        out     TCCR0,r16               ; 




ParameterInitialize:
        ldi     r16,0x00                ; r0 = 0x00
        mov     r0,r16                  ; 

        ldi     r16,0x01                ; r1 = 0x01
        mov     r1,r16                  ; 

        ldi     r16,GateTime_100ms      ; initial gate time=0.1s
        mov     r2,r16                  ; 

        ldi     r16,InputChannel_0      ; initial input channel=ch0
        mov     r3,r16                  ; 

        ldi     r16,Operation_Count     ; initial operation=count
        mov     r4,r16                  ; 




LCDInitialize:
        rcall   Wait22ms

        ldi     r16,0x30                ; function set (I/F 8bit)
        rcall   LCDWrite

        rcall   Wait22ms

        ldi     r16,0x30                ; function set (I/F 8bit)
        rcall   LCDWrite

        rcall   Wait2r2ms

        ldi     r16,0x30                ; function set (I/F 8bit)
        rcall   LCDWrite

        rcall   Wait65us

        ldi     r16,0x20                ; function set (I/F 4bit)
        rcall   LCDWrite

        rcall   Wait65us

        ldi     r16,0x20                ; function set (I/F 4bit, 1/16 duty)
        rcall   LCDWrite
        ldi     r16,0x80
        rcall   LCDWrite

        rcall   Wait65us

        ldi     r16,0x00                ; display control (display off, cursor off, blink off)
        rcall   LCDWrite
        ldi     r16,0x80
        rcall   LCDWrite

        rcall   Wait65us

        ldi     r16,0x00                ; screen clear
        rcall   LCDWrite
        ldi     r16,0x10
        rcall   LCDWrite

        rcall   Wait2r2ms

        ldi     r16,0x00                ; entry mode set (I/D=1, S=0)
        rcall   LCDWrite
        ldi     r16,0x60; 
        rcall   LCDWrite

        rcall   Wait65us

        ldi     r16,0x00                ; display control (display on, cursor off, blink off)
        rcall   LCDWrite
        ldi     r16,0xc0; 
        rcall   LCDWrite

        rcall   Wait65us




        mov     r14,r0                  ; 

KeyPressed:
        cbi     PORTD,0x06              ; LED=0

        sbrc    r14,0x07                ; gate time
        inc     r2                      ; 
        sbrc    r14,0x06                ; input channel
        inc     r3                      ; 
        sbrc    r14,0x05                ; operation
        inc     r4

        mov     r14,r16                 ; old key status = new key status

GateTimeSet:
        mov     r16,r2                  ; gate time
        cpi     r16,GateTime_10000ms+1  ; 
        brcs    ChannelSet              ; 

        ldi     r16,GateTime_10ms       ; if r2 > GateTime_10000ms then r2 = GateTime_10ms
        mov     r2,r16                  ; 

ChannelSet:
        mov     r16,r3                  ; channel
        cpi     r16,InputChannel_2+1    ; 
        brcs    OperationSet            ; 

        ldi     r16,InputChannel_0      ; if r3 > InputChannel_2 then r3 = InputChannel_0
        mov     r3,r16                  ; 

OperationSet:
        mov     r16,r4                  ; operation
        cpi     r16,Operation_Hold+1    ; 
        brcs    StatusDisplay           ; 

        ldi     r16,Operation_Count     ; if r4 > Operation_Hold then r4 = Operation_Count
        mov     r4,r16                  ; 

StatusDisplay:
        rcall   LCDEnable               ;
        rcall   PutStatus               ; 
        rcall   LCDDisable              ;

KeyReleaseCheck:
        rcall   Wait22ms                ;

        in      r16,PINB                ; key status read
        com     r16                     ; r16 = not r16
        andi    r16,0b11110000          ; mask
        or      r14,r16                 ; 
        mov     r14,r16                 ; old key status = new key status
        brne    KeyReleaseCheck         ; 1/2

        mov     r16,r4                  ; 
        cpi     r16,Operation_Count     ; 
        breq    Count                   ; 

Hold:
        rcall   Wait22ms                ; 

        in      r16,PINB                ; new key status read
        com     r16                     ; r16 = not r16
        andi    r16,0b11110000          ; mask
        and     r14,r16                 ; key status compare
        brne    KeyPressed              ; 

        mov     r14,r16                 ; old key status = new key status
        rjmp    Hold                    ; 

Count:
        mov     r16,r2                  ; gate time

        ldi     r23,0x00                ; r23:r22 = 0x0001 = 1
        ldi     r22,0x01                ; 
        cpi     r16,GateTime_10ms       ; 
        breq    CounterClear            ; 

        ldi     r23,0x00                ; r23:r22 = 0x000a = 10
        ldi     r22,0x0a                ; 
        cpi     r16,GateTime_100ms      ; 
        breq    CounterClear            ; 

        ldi     r23,0x00                ; r23:r22 = 0x0064 = 100
        ldi     r22,0x64                ; 
        cpi     r16,GateTime_1000ms     ; 
        breq    CounterClear            ; 

        ldi     r23,0x03                ; r23:r22 = 0x03e8 = 1000
        ldi     r22,0xe8                ; 
;        cpi     r16,GateTime_10000ms    ; 
;        breq    CounterClear            ; 

CounterClear:
        mov     r16,r3                  ; input channel
        lsl     r16                     ; 
        ori     r16,0b00010000          ; LED=0, ENP=0, T0=1, /CLR=0, CH=r3,  1C3=0
        out     PORTD,r16               ; 

        ldi     r31,0x00                ; software counter clear
        ldi     r30,0x00                ; 
        ldi     r29,0x00                ; 
        ldi     r28,0x00                ; 

        ldi     r17,0x00                ; 
        out     TCNT0,r17               ; counter0 clear

        ori     r16,0b00001000          ; LED=0, ENP=0, T0=1, /CLR=1, CH=r3,  1C3=0
        out     PORTD,r16               ; 

        mov     r17,r16                 ; 
        ori     r17,0b00000110          ; LED=0, ENP=0, T0=1, /CLR=1, CH=ch3, 1C3=0
        mov     r15,r17                 ; 

        in      r14,PINB                ; key status read
        com     r14                     ; old key status initialize

        in      r13,TCNT0               ; old counter0 value initialize

        ori     r16,0b01101000          ; LED=1, ENP=1, T0=1, /CLR=1, CH=r3,  1C3=0
        out     PORTD,r16               ; count start

CountLoop:                              ; 10 ms = 128000 step
        rcall   Count10ms               ; 127987

        in      r16,PINB                ; 1     new key status read
        com     r16                     ; 1     r16 = not r16
        andi    r16,0b11110000          ; 1     mask
        and     r14,r16                 ; 1     key status compare
        breq    notkeypressed           ; 1/2   

        rjmp    KeyPressed              ; 2     

notkeypressed:
        mov     r14,r16                 ; 1     old key status = new key status

        sub     r22,r1                  ; 1     loop counter decrement
        sbc     r23,r0                  ; 1     
        mov     r16,r22                 ; 1     loop end check
        or      r16,r23                 ; 1     
        brne    CountLoop               ; 1/2   

        out     PORTD,r15               ; 1     count stop

        rcall   Wait8clk                ; wait

        in      r16,TCNT0               ; new counter0 value read
        cp      r16,r13                 ; if r16<r13 then C=1 else C=0
        adc     r28,r0                  ; software counter increment
        adc     r29,r0                  ; 
        adc     r30,r0                  ; 
        adc     r31,r0                  ; 

        mov     r13,r16                 ; old counter0 value = new counter0 value

        mov     r31,r30                 ; 
        mov     r30,r29                 ; 
        mov     r29,r28                 ; 
        mov     r28,r13                 ; last counter0 value

        ldi     r16,0x04                ; 
countershiftloop:
        lsl     r28                     ; counter x 2
        rol     r29                     ; 
        rol     r30                     ; 
        rol     r31                     ; 

        dec     r16                     ; 
        brne    countershiftloop        ; 

DividerRead:
        ldi     r16,0b00111110          ; LED=0, ENP=1, T0=1, /CLR=1, CH=ch3, 1C3=0
        out     PORTD,r16               ; 

        rcall   Wait8clk                ; wait

        ldi     r17,0x0f                ; max value of external divider 
dividerreadloop:
        sbi     PORTD,0                 ; LED=0, ENP=1, T0=1, /CLR=1, CH=ch3, 1C3=1
        cbi     PORTD,0                 ; LED=0, ENP=1, T0=1, /CLR=1, CH=ch3, 1C3=0

        rcall   Wait8clk                ; wait

        in      r16,TCNT0               ; new counter0 value read
        cp      r16,r13                 ; couner0 value compare
        brne    dividerreadend          ; 

        dec     r17                     ; 
        rjmp    dividerreadloop         ; 

dividerreadend:
        or      r28,r17                 ; count complete




        rcall   LCDEnable               ; 

        rcall   Line1Home               ; 

        ldi     r16,' '                 ; 
        rcall   PutChar                 ; 
        ldi     r16,' '                 ; 
        rcall   PutChar                 ; 

        mov     r27,r31                 ; 
        mov     r26,r30                 ; 
        mov     r25,r29                 ; 
        mov     r24,r28                 ; 

        mov     r16,r2                  ; gate time
        cpi     r16,gateTime_10000ms    ; 
        breq    Gt10000ms               ; 
        cpi     r16,GateTime_1000ms     ; 
        breq    Gt1000ms                ; 
        cpi     r16,GateTime_100ms      ; 
        breq    Gt100ms                 ; 
;        cpi     r16,GateTime_10ms       ; 
;        breq    Gt10ms                  ; 

Gt10ms:
        rcall   Mul10                   ; 
Gt100ms:
        rcall   Mul10                   ;
Gt1000ms:
        rcall   Mul10                   ;
Gt10000ms:

        mov     r12,r0                  ; 
        mov     r11,r0                  ; 

div1G:                                  ; 100M
        ldi     r19,0x3b                ; 
        ldi     r18,0x9a                ; 
        ldi     r17,0xca                ; 
        ldi     r16,0x00                ; 

        rcall   Put1Digit               ; 

div100M:                                ; 10M
        ldi     r19,0x05                ; 
        ldi     r18,0xf5                ; 
        ldi     r17,0xe1                ; 
        ldi     r16,0x00                ; 

        rcall   Put1Digit               ; 

div10M:                                 ; 1M
        ldi     r19,0x00                ; 
        ldi     r18,0x98                ; 
        ldi     r17,0x96                ; 
        ldi     r16,0x80                ; 

        rcall   Put1Digit               ; 

        tst     r11                     ; 
        breq    div1M                   ; 

        ldi     r16,'.'                 ; 
        rcall   PutChar                 ; 

        ldi     r16,'M'                 ; 
        mov     r12,r16                 ; 

div1M:                                  ; 100k
        ldi     r19,0x00                ; 
        ldi     r18,0x0f                ; 
        ldi     r17,0x42                ; 
        ldi     r16,0x40                ; 

        rcall   Put1Digit               ; 

div100k:                                ; 10k
        ldi     r19,0x00                ; 
        ldi     r18,0x01                ; 
        ldi     r17,0x86                ; 
        ldi     r16,0xa0                ; 

        rcall   Put1Digit               ; 

div10k:                                 ; 1k
        ldi     r19,0x00                ; 
        ldi     r18,0x00                ; 
        ldi     r17,0x27                ; 
        ldi     r16,0x10

        rcall   Put1Digit               ; 

        tst     r12                     ; 
        brne    div1k                   ; 

        tst     r11                     ; 
        breq    div1k                   ; 

        ldi     r16,'.'                 ; 
        rcall   PutChar                 ; 

        ldi     r16,'k'                 ; 
        mov     r12,r16                 ; 

div1k:                                  ; 100
        ldi     r19,0x00                ; 
        ldi     r18,0x00                ; 
        ldi     r17,0x03                ; 
        ldi     r16,0xe8

        rcall   Put1Digit               ; 

div100:                                 ; 10
        ldi     r19,0x00                ; 
        ldi     r18,0x00                ; 
        ldi     r17,0x00                ; 
        ldi     r16,0x64

        rcall   Put1Digit               ; 

div10:                                  ; 1
        ldi     r19,0x00                ; 
        ldi     r18,0x00                ; 
        ldi     r17,0x00                ; 
        ldi     r16,0x0a                ; 

        inc     r11                     ; 
        rcall   Put1Digit               ; 

        tst     r12                     ; 
        brne    div1                    ; 

        ldi     r16,'.'                 ; 
        rcall   PutChar                 ; 

        ldi     r16,' '                 ; 
        mov     r12,r16                 ; 

div1:                                   ; 0.1
        mov     r16,r24                 ; 
        subi    r16,0xd0                ; 
        rcall   PutChar                 ; 

        mov     r16,r12                 ;
        rcall   PutChar                 ;

        ldi     r19,HeltzString         ; 
        rcall   Print                   ; 

        rcall   LCDDisable              ;

        rcall   Wait100ms               ; 

        rjmp    Count                   ; 



;------------------------------------------------------------------------------
; Count10ms
; argument     : r13    old counter0 value
; return value : r28, r29, r30, r31
; broken       : r13, r16, r20, r21, r28, r29, r30, r31
; step         : 127987 (include rcall and ret)


Count10ms:

        ldi     r21,0x04                ; 1     0x0419 = 1049
        ldi     r20,0x19                ; 1     
count10msloop:

        ldi     r16,0x24                ; 1     wait 108 step
count10mswaitloop:
        dec     r16                     ; 1     
        brne    count10mswaitloop       ; 1/2   

        nop                             ; 1     wait

        in      r16,TCNT0               ; 1     new counter0 value read
        cp      r16,r13                 ; 1     if r16<r13 then C=1 else C=0
        adc     r28,r0                  ; 1     software counter increment
        adc     r29,r0                  ; 1     
        adc     r30,r0                  ; 1     
        adc     r31,r0                  ; 1     

        mov     r13,r16                 ; 1     old counter0 value = new counter0 value

        sub     r20,r1                  ; 1     loop counter decrement
        sbc     r21,r0                  ; 1     
        mov     r16,r20                 ; 1     loop end check
        or      r16,r21                 ; 1     
        brne    count10msloop           ; 1/2   

        nop                             ; 1     wait

        ret                             ; 4     

;------------------------------------------------------------------------------
; Put1Digit
; argument     : r11    status
;                r20    BCD code
; return value : r11    status
; broken       : r11, r16, r17, r18
; step         : ? (include rcall and ret)


Put1Digit:
        rcall   Div                     ;       

        mov     r16,r20                 ; 1     
        tst     r16                     ; 1     
        brne    putdigit                ; 1/2   

        tst     r11                     ; 1     
        brne    putdigit                ; 1/2   

        ldi     r16,' '                 ; 1     
        rcall   PutChar                 ;       

        ret                             ; 4     

putdigit:
        subi    r16,0xd0                ; 1     value -> ASCII code
        rcall   PutChar                 ;       

        inc     r11                     ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------
; PutStatus
; argument     : r2     gate time
;                r3     channel
;                r4     operation
; return value : no return value
; broken       : r16, r17, r18, r19
; step         : ? (include rcall and ret)


PutStatus:
        rcall   Line2Home               ; 889   

        ldi     r19,ChannelString       ; 1     "Ch-"
        rcall   Print                   ;       

        mov     r16,r3                  ; 1     channel
        subi    r16,0xbf                ; 1     value -> ASCII code
        rcall   PutChar                 ; 894   

        ldi     r19,HoldString          ; 1     " Hold"

        mov     r16,r4                  ; 1     operation
        cpi     r16,Operation_Count     ; 1
        brne    putgt                   ; 1/2

        ldi     r19,GateString          ; 1     " Gate-"
        rcall   Print                   ;       

        mov     r16,r2                  ; 4     gate time

case10ms:
        cpi     r16,GateTime_10ms       ; 1     
        brne    case100ms               ; 1/2   

        ldi     r19,Gt0r01sString       ; 1     " 0.01s"
        rjmp    putgt                   ; 2     

case100ms:
        cpi     r16,GateTime_100ms      ; 1     
        brne    case1000ms              ; 1/2   

        ldi     r19,Gt0r10sString       ; 1     " 0.10s"
        rjmp    putgt                   ; 2     

case1000ms:
        cpi     r16,GateTime_1000ms     ; 1     
        brne    case10000ms             ; 1/2   

        ldi     r19,Gt1r00sString       ; 1     " 1.00s"
        rjmp    putgt                   ; 2     

case10000ms:
        ldi     r19,Gt10r00sString      ; 1     "10.00s"

putgt:
        rcall   Print                   ; 

        ret                             ; 4     

;------------------------------------------------------------------------------
; Print
; argument     : r19    pointer
; return value : no return value
; broken       : r16, r17, r18, r19
; step         : ? (include rcall and ret)


printloop:
        mov     r18,r16                 ; 1     save argument
        andi    r16,0xf0                ; 1     
        ori     r16,0x04                ; 1     
        rcall   LCDWrite                ; 24    

        mov     r16,r18                 ; 1     
        swap    r16                     ; 1     
        andi    r16,0xf0                ; 1     
        ori     r16,0x04                ; 1     
        rcall   LCDWrite                ; 24    

        rcall   Wait65us                ; 832   

Print:
        out     EEAR,r19                ; 1     EEPROM read address set
        inc     r19                     ; 1     EEPROM read address increment
        sbi     EECR,0x00               ; 2     EEPROM Read strobe set
        in      r16,EEDR                ; 1     EEPROM data read

        cpi     r16,0x00                ; 1     end code check
        brne    printloop               ; 1/2   

        ret                             ; 4     

;------------------------------------------------------------------------------
; PutChar
; argument     : r16    ASCII code
; return value : no return value
; broken       : r16, r17, r18
; step         : 894 (include rcall and ret)


PutChar:
        mov     r18,r16                 ; 1     save argument
        andi    r16,0xf0                ; 1     
        ori     r16,0x04                ; 1     
        rcall   LCDWrite                ; 24    

        mov     r16,r18                 ; 1     
        swap    r16                     ; 1     
        andi    r16,0xf0                ; 1     
        ori     r16,0x04                ; 1     
        rcall   LCDWrite                ; 24    

        rcall   Wait65us                ; 832   

        ret                             ; 4     

;------------------------------------------------------------------------------
; Line1Home
; argument     : no argument
; return value : no return value
; broken       : r16, r17
; step         : 889 (include rcall and ret)


Line1Home:
        ldi     r16,0x80                ; 1     
        rcall   LCDWrite                ; 24    
        ldi     r16,0x00                ; 1     DD RAM address 0x00 set
        rcall   LCDWrite                ; 24    

        rcall   Wait65us                ; 832   

        ret                             ; 4     

;------------------------------------------------------------------------------
; Line2Home
; argument     : no argument
; return value : no return value
; broken       : r16, r17
; step         : 889 (include rcall and ret)


Line2Home:
        ldi     r16,0xc0                ; 1     
        rcall   LCDWrite                ; 24    
        ldi     r16,0x00                ; 1     DD RAM address 0x40 set
        rcall   LCDWrite                ; 24    

        rcall   Wait65us                ; 832   

        ret                             ; 4     

;------------------------------------------------------------------------------
; ScreenClear
; argument     : no argument
; return value : no return value
; broken       : r16, r17
; step         : 21917 (include rcall and ret)


ScreenClear:
        ldi     r16,0x00                ; 1     
        rcall   LCDWrite                ; 24    
        ldi     r16,0x10                ; 1     screen clear
        rcall   LCDWrite                ; 24    

        rcall   Wait2r2ms               ; 21860 

        ret                             ; 4     

;------------------------------------------------------------------------------
; LCDWrite
; argument     : r16    d7  d6  d5  d4  d3  d2  d1  d0
;                data   DB7 DB6 DB5 DB4 x   RS  x   x 
; return value : no return value
; broken       : r16, r17
; step         : 24 (include rcall and ret)


LCDWrite:
        in      r17,PORTB               ; 1     current value
        andi    r17,0b00000011          ; 1     mask

        andi    r16,0b11110100          ; 1     mask

        or      r16,r17                 ; 1     compose

        out     PORTB,r16               ; 1     new value

        nop                             ; 1     wait
        nop                             ; 1     wait

        sbi     PORTB,0x03              ; 2     E=1

        nop                             ; 1     wait
        nop                             ; 1     wait
        nop                             ; 1     wait
        nop                             ; 1     wait
        nop                             ; 1     wait
        nop                             ; 1     wait

        cbi     PORTB,0x03              ; 2     E = 0

        ret                             ; 4     

;------------------------------------------------------------------------------
; LCDEnable
; argument     : no argument
; return value : no return value
; broken       : r16
; step         : 11 (include rcall and ret)


LCDEnable:
        ldi     r16,0b11111111          ; 1     portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     DDRB,r16                ; 1     i/o    out out out out out out out out

        ldi     r16,0b00000000          ; 1     portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     PORTB,r16               ; 1     data   0   0   0   0   0   0   0   0

        ret                             ; 4     

;------------------------------------------------------------------------------
; LCDDisable
; argument     : no argument
; return value : no return value
; broken       : r16
; step         : 11 (include rcall and ret)


LCDDisable:
        ldi     r16,0b00001111          ; 1     portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     DDRB,r16                ; 1     i/o    in  in  in  in  out out out out

        ldi     r16,0b11110000          ; 1     portb  d7  d6  d5  d4  d3  d2  d1  d0
        out     PORTB,r16               ; 1     data   1   1   1   1   0   0   0   0

        ret                             ; 4     

;------------------------------------------------------------------------------
; Mul10
; argument     : r27:r26:r25:r24 = HH:HL:LH:LL
; return value : r27:r26:r25:r24 = HH:HL:LH:LL
; broken       : r16, r17, r18, r19, r24, r25, r26, r27
; step         : 27 (include rcall and ret)


Mul10:
        mov     r16,r24                 ; 1     
        mov     r17,r25                 ; 1     
        mov     r18,r26                 ; 1     
        mov     r19,r27                 ; 1     

        lsl     r24                     ; 1     x 2
        rol     r25                     ; 1     
        rol     r26                     ; 1     
        rol     r27                     ; 1     

        lsl     r24                     ; 1     x 4
        rol     r25                     ; 1     
        rol     r26                     ; 1     
        rol     r27                     ; 1     

        add     r24,r16                 ; 1     x 5
        adc     r25,r17                 ; 1     
        adc     r26,r18                 ; 1     
        adc     r27,r19                 ; 1     

        lsl     r24                     ; 1     x 10
        rol     r25                     ; 1     
        rol     r26                     ; 1     
        rol     r27                     ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------
; Div
; argument     : r27:r26:r25:r24 = divident  (HH:HL:LH:LL)
;                r19:r18:r17:r16 = divisor   (HH:HL:LH:LL)
; return value : r23:r22:r21:r20 = modulus   (HH:HL:LH:LL)
;                r27:r26:r25:r24 = remainder (HH:HL:LH:LL)
; broken       : r20, r21, r22, r23, r24, r25, r26, r27
; step         : ? (include rcall and ret)


Div:
        ldi     r20,0x00                ; 1     
        ldi     r21,0x00                ; 1     
        ldi     r22,0x00                ; 1     
        ldi     r23,0x00                ; 1     

divloop:
        sub     r24,r16                 ; 1     
        sbc     r25,r17                 ; 1     
        sbc     r26,r18                 ; 1     
        sbc     r27,r19                 ; 1     
        brcs    divend                  ; 1     

        add     r20,r1                  ; 1     
        adc     r21,r0                  ; 1     
        adc     r22,r0                  ; 1     
        adc     r23,r0                  ; 1     
        rjmp    divloop                 ; 1     

divend:
        add     r24,r16                 ; 1     
        adc     r25,r17                 ; 1     
        adc     r26,r18                 ; 1     
        adc     r27,r19                 ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------
; Wait8clk
; argument     : no argument
; return value : no return value
; broken       : no broken registers
; step         : 8 (include rcall and ret)


Wait8clk:
        nop                             ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------
; Wait65us
; argument     : no argument
; return value : no return value
; broken       : r16
; step         : 832 (include rcall and ret)


Wait65us:                               ; wait 65us include rcall+ret
                                        ; 3+1+(1+1+1+1+2)*(0x89-1)+(1+1+1+1+1)+1+1+1+4=832

        ldi     r16,0x89                ; 1     
wait65usloop:
        nop                             ; 1     
        nop                             ; 1     
        nop                             ; 1     
        dec     r16                     ; 1     
        brne    wait65usloop            ; 1/2   

        nop                             ; 1     
        nop                             ; 1     
        nop                             ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------
; Wait2r2ms
; argument     : no argument
; return value : no return value
; broken       : r16, r17, r18
; step         : 21860 (include rcall and ret)


Wait2r2ms:                              ; wait 2.2ms include rcall+ret
                                        ; 3+1+1+(1+1+1+1+2)*(0xe3a-1)+(1+1+1+1+1)+4=21860

        ldi     r17,0x0e                ; 1     
        ldi     r16,0x3a                ; 1     
wait2r2msloop:
        subi    r16,0x01                ; 1     
        sbci    r17,0x00                ; 1     
        mov     r18,r16                 ; 1     
        or      r18,r17                 ; 1     
        brne    wait2r2msloop           ; 1/2   

        ret                             ; 4     

;------------------------------------------------------------------------------
; Wait22ms
; argument     : no argument
; return value : no return value
; broken       : r16, r17, r18
; step         : 281600 (include rcall and ret)


Wait22ms:                               ; wait 22ms include rcall+ret
                                        ; 3+1+1+(1+1+1+1+2)*(0xb754-1)+(1+1+1+1+1)+4=281600

        ldi     r17,0xb7                ; 1     
        ldi     r16,0x54                ; 1     
wait22msloop:
        subi    r16,0x01                ; 1     
        sbci    r17,0x00                ; 1     
        mov     r18,r16                 ; 1     
        or      r18,r17                 ; 1     
        brne    wait22msloop            ; 1/2   

        ret                             ; 4     

;------------------------------------------------------------------------------
Wait100ms:                              ; wait 100ms include rcall+ret
                                        ; 3+1+1+1+(1+1+1+1+1+1+1+1+2)*(0x1f3ff-1)+(1+1+1+1+1+1+1+1+1)+1+4=1280000

        ldi     r18,0x01                ; 1     
        ldi     r17,0xf3                ; 1     
        ldi     r16,0xff                ; 1     
wait100msloop:
        nop                             ; 1     
        nop                             ; 1     
        subi    r16,0x01                ; 1     
        sbci    r17,0x00                ; 1     
        sbci    r18,0x00                ; 1     
        mov     r19,r16                 ; 1     
        or      r19,r17                 ; 1     
        or      r19,r18                 ; 1     
        brne    wait100msloop           ; 1/2   

        nop                             ; 1     

        ret                             ; 4     

;------------------------------------------------------------------------------

        .eseg

ChannelString:
        .db     "Ch-", 0x00             ; 4

HoldString:
        .db     " Hold", 0x00           ; 6

GateString:
        .db     " Gate-", 0x00          ; 7

Gt0r01sString:
        .db     " 0.01s", 0x00          ; 7

Gt0r10sString:
        .db     " 0.10s", 0x00          ; 7

Gt1r00sString:
        .db     " 1.00s", 0x00          ; 7

Gt10r00sString:
        .db     "10.00s", 0x00          ; 7

HeltzString:
        .db     "Hz", 0x00              ; 3

;------------------------------------------------------------------------------
